@@ -28,7 +28,7 @@ def indent(text: str, indent: int = 2) -> str:
 class DipDupException(Exception):
     message: str
 
-    def __init__(self, *args) -> None:
+    def __init__(self, *args: Any) -> None:
         super().__init__(self.message, *args)
 
 
@@ -124,7 +124,7 @@ class MigrationRequiredError(DipDupError):
             ],
             headers=['', 'spec_version', 'DipDup version'],
         )
-        reindex = '\n\n' + _tab + ReindexingRequiredError(ReindexingReason.MIGRATION).help() if self.reindex else ''
+        reindex = '\n\n' + _tab + ReindexingRequiredError(ReindexingReason.migration).help() if self.reindex else ''
         return f"""
             Project migration required!
 
@@ -250,7 +250,7 @@ class IndexAlreadyExistsError(DipDupError):
 class InvalidDataError(DipDupError):
     """Failed to validate datasource message against generated type class"""
 
-    type_cls: Type
+    type_cls: Type[Any]
     data: Any
     parsed_object: Any
 
@@ -291,8 +291,8 @@ class CallbackTypeError(DipDupError):
     name: str
 
     arg: str
-    type_: Type
-    expected_type: Type
+    type_: Type[Any]
+    expected_type: Type[Any]
 
     def _help(self) -> str:
         return f"""
@@ -306,26 +306,6 @@ class CallbackTypeError(DipDupError):
         """
 
 
-# TODO: Drop in next major version
-@dataclass(frozen=True, repr=False)
-class DeprecatedHandlerError(DipDupError):
-    """Default handlers need to be converted to hooks"""
-
-    def _help(self) -> str:
-        return """
-            Default handlers have been deprecated in favor of hooks in DipDup 3.0.
-
-              * `handlers/on_rollback.py` -> `hooks/on_rollback.py`
-              * `handlers/on_configure.py` -> `hooks/on_restart.py`
-              * [none] -> `hooks/on_reindex.py`
-
-            Perform the following actions:
-
-              1. If you have any custom logic implemented in default handlers move it to corresponding hooks from the table above.
-              2. Remove default handlers from project.
-        """
-
-
 @dataclass(frozen=True, repr=False)
 class HasuraError(DipDupError):
     """Failed to configure Hasura instance"""
